package service

import (
	"context"

	"git.pbiernat.dev/egommerce/basket-service/internal/app/event"
	"git.pbiernat.dev/egommerce/basket-service/pkg/amqp"
	"git.pbiernat.dev/egommerce/basket-service/pkg/fluentd"
	"github.com/jackc/pgx/v5/pgxpool"
	base "github.com/streadway/amqp"
)

type BasketService struct {
	dbConn *pgxpool.Pool
	ebCh   *base.Channel
	log    *fluentd.Logger
}

func NewBasketService(dbConn *pgxpool.Pool, chn *base.Channel, log *fluentd.Logger) *BasketService {
	return &BasketService{dbConn, chn, log}
}

func (s *BasketService) Get(basketID string) (interface{}, error) {
	// basket := new(def.Basket)
	var createdAt interface{}
	row := s.dbConn.QueryRow(context.Background(), `SELECT created_at FROM "basket-svc".basket WHERE id=$1`, basketID)
	err := row.Scan(createdAt)
	if err != nil {
		s.log.Log("GET CART: %v", err)
		return nil, err
	}

	return createdAt, nil
}

func (s *BasketService) Create(basketID string) error {
	sql := `INSERT INTO "basket-svc".basket(id) VALUES($1)`
	_, err := s.dbConn.Exec(context.Background(), sql, basketID)
	// s.dbConn.Query(context.Background(), `INSERT INTO "basket-svc".basket(id) VALUES($1)`, basketID)

	return err
}

func (s *BasketService) Checkout(reqID, basketID string) (string, error) {
	s.log.Log("Creating initial order from basket#:%s", basketID)

	msg := &event.BasketCheckoutEvent{Event: event.NewEvent(reqID), BasketID: basketID}
	amqp.Publish(s.ebCh, "api-events", "basket.order.basketCheckout", msg) // fixme probably redundant in eventName in routingKey

	return basketID, nil
}
